;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;  System       : 78K/0 EEPROM emulation
;  File name    : EEPROMEx.asm
;  Version name : 1.00
;  Target CPU   : uPD78F0547(78K0/KF2)
;  Last updated : 2005/09/08
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

PUBLIC	ucEEPROMInitEx
PUBLIC	ucEEPROMWriteEx
PUBLIC	ucEEPROMReadEx
PUBLIC	ucEEPROMEraseEx
PUBLIC	ucEEPROMChangeEx
PUBLIC	ucEEPROMGetRemainEx

EXTRN	_FlashStart
EXTRN	_FlashEnd
EXTRN	_FlashEnv
EXTRN	_FlashBlockErase
EXTRN	_FlashBlockBlankCheck
EXTRN	_CheckFLMD
EXTRN	_EEPROMWrite
EXTRN	_FlashWordWrite

$RESET(BANKUSE)

$IF(DEBUG_EEPROM)
$include(user.inc)
$ELSE
;---------------------------------------------------------------------
;	Block numbers of blocks used as EEPROM 	## User settings ##
;---------------------------------------------------------------------
EE_TBL1	CSEG
tEEPROM_BLOCK:					;ex) Block numbers 32,33,34 are used.
	DB	32				;    (Set blocks must be consecutive.
	DB	33				;     Set the blocks so as not to overlap the program area.)
	DB	34
tEEPROM_BLOCK_END:

tEEPROM_BANK:					;ex) Bank numbers 0 is used.(The product without a ROM bank sets up 0. )
	DB	0
	DB	0
	DB	0
tEEPROM_BANK_END:

;---------------------------------------------------------------------
;	Data length of write data 		## User settings ##
;---------------------------------------------------------------------
	DATALENGTH	EQU	4		;ex) data length

;---------------------------------------------------------------------
;	Maximum of a data number 		## User settings ##
;---------------------------------------------------------------------
	DATANOMAX	EQU	10		;ex) data number 0 to 10

;---------------------------------------------------------------------
;	Please comment out the product without a ROM bank. 
;						## User settings ##
;---------------------------------------------------------------------
	$SET(BANKUSE)				;ex) A ROM bank is used

;---------------------------------------------------------------------
;	The re-try number of times at the time of a write error 
;						## User settings ##
;---------------------------------------------------------------------
	WRITENUM	EQU	2		;ex) 2times

;---------------------------------------------------------------------
;	The re-try number of times at the time of an elimination error
;						## User settings ##
;---------------------------------------------------------------------
	ERASENUM	EQU	2		;ex) 2times

;---------------------------------------------------------------------
;	Elimination operation existence at the time of block change
;						## User settings ##
;---------------------------------------------------------------------
	$SET		(AUTOERASE)		;ex) It eliminates
$ENDIF

;---------------------------------------------------------------------
;	Other EQU settings
;---------------------------------------------------------------------
	TRUE		EQU	00H		;Normal
	FALSE		EQU	0FFH		;Abnormal
	
	NMLBLK		EQU	01H		;Noemal end(Block eror)
	ERRPRM		EQU	0F4H		;Param Error
	ERRFULL		EQU	0F5H		;Data full
	ERRWRITE	EQU	0F6H		;Write error
	ERRNODAT	EQU	0F7H		;Reading data nothing
	ERRNOBLA	EQU	0F8H		;Block nothing finished with elimination
	ERRNOERS	EQU	0F9H		;With no eliminable block
	ERRNOUSE	EQU	0FAH		;With no effective block
	ERRALL		EQU	0FBH		;Prohibition of all block use
	ERRBLK		EQU	0FCH		;Block eror
	ERRDEV		EQU	0FDH		;Device error
	ERRINIT		EQU	0FEH		;Initialization error
		
	LIB_NORMAL_END	EQU	00H		;Normal completion(== constant definition:TRUE)
	LIB_BLKCHK_ERR	EQU	1BH		;Blank check error
	LIB_BLANK_ERR	EQU	1EH		;Blank error
	LIB_STATE_INT	EQU	1FH		;State of interruption

	DATATOP		EQU	8
	DATA_NO_MAX	EQU	(DATANOMAX+1)	;Number of write data
	WRITE_DATA_LEN	EQU	(((((DATALENGTH+1)/4)+1)*4)-1)	;Data length{Data which buried the insufficiency of a 4-byte boundary{Delimiter

	EEPROM_BLOCK_NO	EQU	(tEEPROM_BLOCK_END - tEEPROM_BLOCK)	;Number of blocks used as EEPROM

	DATA_OF_BLOCK	EQU	((0400H-DATATOP)/(WRITE_DATA_LEN+1))
	DATA_END_OFFSET	EQU	((DATA_OF_BLOCK*(WRITE_DATA_LEN+1))+DATATOP)

;---------------------------------------------------------------------
;	Entry RAM settings
;---------------------------------------------------------------------
EE_SADR	DSEG	saddr
	b100EntryRAM:	DS	100		;Entry RAM for flash firmware control
	
;---------------------------------------------------------------------
;	Other RAM settings
;---------------------------------------------------------------------
EE_RAM	DSEG
	bDataBuf:	DS	WRITE_DATA_LEN+1;Calculate internal buffer setting value using following expression
						;(((maximum length of write data+1)/4)+1)*4)
						;Round off remainder of division
						;Set 8, when result is 8 or less

						;for User access processing
	dUseBlockTBLNo:	DS	1		;RAM for storing block being used during EEPROM read/write processing
	dDstBlockTBLNo:	DS	1		;RAM for storing transition destination block number
	d2SrcAdr:	DS	2		;RAM for storing data transfer source address during block transition
	d2DstAdr:	DS	2		;RAM for storing data transfer destination address during block transition
	d2SetAdr:	DS	2		;RAM for storing data transfer destination address
	dBankNo:	DS	1		;Bank No.
	dFlag:		DS	1		;General-purpose flag
	ci:		DS	1		;General-purpose loop counter
	cj:		DS	1		;General-purpose loop counter
	
						;for EEPROMEmulation control processing
	d2SearchAdr:	DS	2		;RAM for storing address during EEPROM data search

						;for Self library control processing
	WordAddr:				;Write data structure
	WriteAddress:	DS	2		;Write address
	WriteBnak:	DS	1		;Write Bank No.

EE_PROG	CSEG

;-------------------------------------------------------------------
; Processing name : ucEEPROMInitEx(User access processing)
; Input           : None
; Output          : C=Normal(TRUE:00H)
;                     Noemal end(Block error)(NMLBLK:01H)
;                     Prohibition of all block use(ERRALL:FBH)
;                     Device error(ERRDEV:FDH)
;                     Initialization error(ERRINIT:FEH)
; Summary         : EEPROM emulation initialize
;-------------------------------------------------------------------
ucEEPROMInitEx:
	PUSH	HL				;Register evacuation
	PUSH	DE
	PUSH	AX
	
	CALL	!ParamCheck
	CMP	A,#TRUE
	BZ	$EP_INIT_10
	MOV	C,#ERRPRM
	BR	EP_INIT_E
	
;block erase process
EP_INIT_10:
	MOV	A,#0FFH				;effective Block table number
	MOV	!dUseBlockTBLNo,A
	MOV	A,#0
	MOV	!dDstBlockTBLNo,A		;The candidate for processing Block table number
	MOV	!dFlag,A			;Clear Block eror flag
	
EP_INIT_11:
	MOV	A,!dDstBlockTBLNo		;Block table number
	CALL	!BankSet			;Bank set
	CALL	!BlockToAddress			;Address translation
	MOVW	HL,AX				;  --> HL
	
	CALL	!UseProhibitionBlock		;It investigates in a use prohibition block
	CMP	A,#TRUE				;if Use prohibition block?
	BZ	$EP_INIT_15			;  then jump
	
	CALL	!EffectiveBlock			;It investigates in an effective block
	CMP	A,#FALSE			;if Not effective block?
	BZ	$EP_INIT_12			;  then jump
	
	MOV	A,!dUseBlockTBLNo
	CMP	A,#0FFH				;if finished with effective block discovery?
	BNZ	$EP_INIT_12			;  then jump
	
	MOV	A,!dDstBlockTBLNo
	MOV	!dUseBlockTBLNo,A		;effective block no.
	BR	$EP_INIT_15
	
EP_INIT_12:
	CALL	!SelfFlashModeOn		;Sets flash memory mode
	CALL	!ucSelfFlashInitialize		;Initializes flash memory
	CMP	A,#TRUE				;if Noemal end?
	BZ	$EP_INIT_13			;  then jump
	BR	EP_INIT_IN			;  else jump
	
EP_INIT_13:
	MOV	A,#ERASENUM+1			;erase counter initialization
	MOV	!ci,A
	
EP_INIT_14:
	MOV	A,!dDstBlockTBLNo
	MOV	C,A
	MOVW	HL,#tEEPROM_BANK+0		;bank table
	MOV	A,[HL+C]
	XCH	A,X				;Bank set-->X
	MOVW	HL,#tEEPROM_BLOCK+0		;block table
	MOV	A,[HL+C]			;block no.-->A
	XCH	A,X
	CALL	!ucSelfFlashBlockErase		;Erases block
	CMP	A,#TRUE				;if Noemal end?
	BZ	$EP_INIT_15			;  then jump
	
	MOV	A,!ci
	DEC	A				;counter-1
	MOV	!ci,A
	BNZ	$EP_INIT_14			;if counter!=0 then jump
	
	MOV	A,#1
	MOV	!dFlag,A			;Block eror flag set
	
	MOV	A,!dDstBlockTBLNo		;Block table number set
	CALL	!ucEEPROMUnableEx		;Prohibition of the Brock use
	CMP	A,#TRUE				;if Noemal end?
	BZ	$EP_INIT_15			;  then jump
	BR	EP_INIT_DE			;  else jump
	
EP_INIT_15:
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	
	MOV	A,!dDstBlockTBLNo
	INC	A				;Block table number+1
	CMP	A,#EEPROM_BLOCK_NO		;if finish?
	BNC	$EP_INIT_20			;  then jump
	
	MOV	!dDstBlockTBLNo,A		;Block table number save
	BR	$EP_INIT_11			;loop

;effective block set process
EP_INIT_20:
	MOV	A,!dUseBlockTBLNo		;effective Block table number
	CMP	A,#0FFH				;if effective block find?
	BNZ	$EP_INIT_24			;  then jump
	
	MOV	A,#0
	MOV	!dDstBlockTBLNo,A		;Block table number
EP_INIT_21:
	MOV	A,!dDstBlockTBLNo		;block No.
	CALL	!BankSet			;Bank set
	CALL	!BlockToAddress			;Address translation
	MOVW	HL,AX				;  --> HL
	
	CALL	!BlankBlock			;It investigates in an eliminated block
	CMP	A,#TRUE				;if eliminated block?
	BNZ	$EP_INIT_23			;  else jump
	
	MOVW	AX,#5555H			;effective flag data
	MOVW	!bDataBuf+0,AX
	MOVW	!bDataBuf+2,AX
	
	CALL	!SelfFlashModeOn		;Sets flash memory mode
	CALL	!ucSelfFlashInitialize		;Initializes flash memory
	CMP	A,#TRUE				;if Noemal end?
	BNZ	$EP_INIT_IN			;  else jump
	
	MOVW	AX,HL
	MOVW	DE,AX				;Writes start address
	MOV	A,!dDstBlockTBLNo
	MOV	C,A
	MOVW	HL,#tEEPROM_BANK+0
	MOV	A,[HL+C]			;Bank number seetting
	MOV	B,#4-1				;Data number setting
	CALL	!ucSelfFlashEEPROMWrite		;Sets block being used as invalid
	CMP	A,#TRUE
	BNZ	$EP_INIT_22
	
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	BR	$EP_INIT_24
	
EP_INIT_22:
	MOV	A,!dDstBlockTBLNo		;Block table number set
	CALL	!ucEEPROMUnableEx		;Prohibition of the Brock use
	CMP	A,#TRUE				;if Noemal end?
	BNZ	$EP_INIT_DE			;  else jump
	
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	
	MOV	A,#1
	MOV	!dFlag,A			;Block eror flag set
	
EP_INIT_23:
	MOV	A,!dDstBlockTBLNo
	INC	A				;Block table number+1
	CMP	A,#EEPROM_BLOCK_NO		;if finish?
	BNC	$EP_INIT_AL			;  then jump
	
	MOV	!dDstBlockTBLNo,A		;Block table number save
	BR	$EP_INIT_21			;loop
	
EP_INIT_24:
	MOV	A,!dFlag			;Block eror flag
	CMP	A,#0				;if Block eror?
	BNZ	$EP_INIT_BL			;  then jump

	MOV	C,#TRUE				;return value=Normal
	BR	$EP_INIT_E

;I
EP_INIT_IN:
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	MOV	C,#ERRINIT			;return value=Initialization error
	BR	$EP_INIT_E			;jump to end
	
EP_INIT_DE:
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	MOV	C,#ERRDEV			;return value=Device error
	BR	$EP_INIT_E			;jump to end
	
EP_INIT_BL:
	MOV	C,#NMLBLK			;return value=Normal(Block eror)
	BR	$EP_INIT_E			;jump to end
	
EP_INIT_AL:
	MOV	C,#ERRALL			;return value=Prohibition of all block use
	
EP_INIT_E:

	POP	AX				;Register restoration
	POP	DE
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucEEPROMReadEx(User access processing)
; Input           : X=Read data number
;	          : DE=Read data storage address
; Output          : C=Normal completion(TRUE:00H)
;                     Reading data nothing(ERRNODAT:F7H)
;                     With no effective block(ERRNOUSE:FAH)
; Summary         : Data with the specified number is read from EEPROM to the storage address.
;	          : 1.Searches for blocks being used as EEPROM.
;	          : 2.Searches for latest datafs address among valid blocks.
;	          : 3.Reads latest data from searched address.
;-------------------------------------------------------------------
ucEEPROMReadEx:
	PUSH	HL
	PUSH	DE
	PUSH	AX
	
	MOVW	AX,SP
	MOVW	HL,AX
	
	CALL	!ParamCheck
	CMP	A,#TRUE
	BZ	$EP_READ_0
	MOV	C,#ERRPRM
	BR	$EP_READ_E
	
EP_READ_0:
	CALL	!ucEEPROMUseBlockSearchEx	;Searches for blocks currently used as EEPROM
	CMP	A,#FALSE
	BZ	$EP_READ_NU			;Abnormal completion
	
	XCH	A,X
	MOV	A,[HL]
	XCH	A,X				;A=Block table number
						;X=Data No.
	CALL	!usEEPROMDataSearchEx		;Obtains address of latest data of specified data number
	CMPW	AX,#0000H
	BZ	$EP_READ_ND			;Abnormal completion
	
	INCW	AX
	MOVW	BC,AX
	
	MOV	A,[HL+2]
	XCH	A,X
	MOV	A,[HL+3]
	MOVW	DE,AX				;read data storing adress
	MOVW	AX,BC
	MOVW	HL,AX				;
	MOV	C,#0				;Loop Counter
EP_READ_1:
	MOV	A,C
	CMP	A,#DATALENGTH			;Loops from 0 to data length
	BNC	$EP_READ_2
	
	MOV	A,[HL]
	MOV	[DE],A				;Stores data length amount of latest data
	INCW	HL
	INCW	DE
	
	INC	C				;Loop counter + 1
	BR	$EP_READ_1
	
EP_READ_2:
	MOV	A,L				;lower addres
	AND	A,#03H				;lower 2bit
	CMP	A,#03H				;if last offset address=+3or+7or+11or+15...
	BNZ	$EP_READ_TR			;  else jump
	
	DECW	DE				;address decriment
	MOV	A,[HL]
	MOV	[DE],A				;delimiter set
	
EP_READ_TR:
	MOV	C,#TRUE				;Return value = Normal completion (TRUE)
	BR	$EP_READ_E
	
EP_READ_ND:
	MOV	C,#ERRNODAT			;Return value = no data (ERRNODAT)
	BR	$EP_READ_E
	
EP_READ_NU:
	MOV	C,#ERRNOUSE			;Return value = With no effective block (ERRNOUSE)
	
EP_READ_E:
	POP	AX
	POP	DE
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucEEPROMWriteEx(user access processing)
; Input		  : X=Write data Number
;		  : DE=Write data storage address
; Output	  : C=Normal completion(TRUE:00H)
;                     Data full(ERRFULL:F5H)
;                     Write error(ERRWRITE:F6H)
;                     With no effective block(ERRNOUSE:FAH)
;                     Initialization error(ERRINIT:FEH)
; Summary	  : Writes data with specified number from the storage address to EEPROM.
;		  : 1.Searches for blocks being used as EEPROM.
;		  : 2:Searches for addresses to enable writing to valid block.
;		  : 3:Creates write data.
;		  : 4:Writes to valid blocks.
;-------------------------------------------------------------------
ucEEPROMWriteEx:
	PUSH	HL
	PUSH	DE
	PUSH	AX
	
	CALL	!ParamCheck
	CMP	A,#TRUE
	BZ	$EP_WRITE_00
	MOV	C,#ERRPRM
	BR	EP_WRITE_E
	
EP_WRITE_00:
	CALL	!ucEEPROMUseBlockSearchEx	;Searches for blocks being used as EEPROM
	CMP	A,#FALSE			;if Abnormal end?
	BNZ	$EP_WRITE_10			;  else jump
	BR	EP_WRITE_NU			;  then jump
	
EP_WRITE_10:
	MOV	!dUseBlockTBLNo,A		;Block table number save
	MOVW	AX,#bDataBuf+0
	ADDW	AX,#WRITE_DATA_LEN
	MOVW	DE,AX				;last data storing address
	MOV	A,#0FFH				;set FFh
	MOV	[DE],A
	DECW	DE
	MOV	[DE],A
	DECW	DE
	MOV	[DE],A
	DECW	DE
	MOV	[DE],A
	
	MOVW	AX,SP
	MOVW	HL,AX				;read SP
	
	MOV	A,[HL]
	MOV	!bDataBuf+0,A			;Data No. storing
	
	MOV	A,[HL+2]
	XCH	A,X
	MOV	A,[HL+3]
	MOVW	DE,AX				;write data storing addres
	
	MOVW	HL,#bDataBuf+0			;buffer data write address
	INCW	HL
	
	MOV	C,#0				;Loop Counter
EP_WRITE_11:
	MOV	A,[DE]				;read data
	MOV	[HL],A				;set to buffer
	INCW	HL
	INCW	DE
	
	INC	C				;Loop counter + 1
	MOV	A,C
	CMP	A,#DATALENGTH			;Loops from 0 to data length
	BC	$EP_WRITE_11
	
	MOVW	AX,#bDataBuf+0
	ADDW	AX,#WRITE_DATA_LEN-1
	MOVW	HL,AX
	MOV	A,[HL]				;read last data
	MOV	X,#00H
	XCH	A,X
	MOV	[HL],A				;delimiter(00H) set
	INCW	HL
	XCH	A,X
	MOV	[HL],A				;last data move
	
EP_WRITE_20:
	MOV	A,!dUseBlockTBLNo
	MOV	C,A
	
	CALL	!BlockToAddress			;
	ADDW	AX,#DATA_END_OFFSET		;add offset
	MOVW	!d2DstAdr,AX			;last address save
	
	MOVW	HL,#tEEPROM_BANK+0
	MOV	A,[HL+C]
	MOV	!dBankNo,A			;Bank No. save
	
	MOV	A,#WRITENUM+1			;counter initialization
	MOV	!ci,A
	
	MOV	A,!dUseBlockTBLNo
	CALL	!usEEPROMWriteTopSearchEx
	MOVW	!d2SetAdr,AX			;Write address save
	
	CALL	!SelfFlashModeOn		;Sets flash memory mode
	CALL	!ucSelfFlashInitialize		;Initializes flash memory
	CMP	A,#TRUE				;if Noemal end
	BNZ	$EP_WRITE_IN			;  else jump
	
EP_WRITE_21:
	MOVW	AX,!d2SetAdr			;Write address save
	CMP	A,!d2DstAdr+1			;if any space?
	BC	$EP_WRITE_22			;  then jump
	XCH	A,X
	CMP	A,!d2DstAdr+0			;if any space?
	BNC	$EP_WRITE_FU			;  else jump
	
	XCH	A,X
EP_WRITE_22:
	MOVW	DE,AX				;DE=Write address
	MOV	B,#WRITE_DATA_LEN		;B=Data byte num
	MOV	A,!dBankNo			;A=Bank No.
	
	CALL	!ucSelfFlashEEPROMWrite		;Writes created write data to EEPROM
	CMP	A,#TRUE				;if Noemal end
	BZ	$EP_WRITE_TR			;  then jump
	
	CMP	A,#LIB_BLANK_ERR		;if blank error
	BZ	$EP_WRITE_23
	
	MOV	A,!ci
	DEC	A				;if retry over
	BZ	$EP_WRITE_WE			;  then jump
	
	MOV	!ci,A
EP_WRITE_23:
	MOVW	AX,DE
	ADDW	AX,#WRITE_DATA_LEN+1		;address update
	MOVW	!d2SetAdr,AX			;Write address save
	BR	$EP_WRITE_21
	
EP_WRITE_NU:
	MOV	C,#ERRNOUSE			;Return value = With no effective block (ERRNOUSE)
	BR	$EP_WRITE_E
	
EP_WRITE_IN:
	MOV	C,#ERRINIT			;Return value = Initialization error (ERRINIT)
	BR	$EP_WRITE_MO
	
EP_WRITE_FU:
	MOV	C,#ERRFULL			;Return value = Data full (ERRFULL)
	BR	$EP_WRITE_MO
	
EP_WRITE_WE:
	MOV	C,#ERRWRITE			;Return value = Write error (ERRWRITE)
	BR	$EP_WRITE_MO
	
EP_WRITE_TR:
	MOV	C,#TRUE				;Return value = Normal completion (TRUE)
	
EP_WRITE_MO:
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	
EP_WRITE_E:
	POP	AX
	POP	DE
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucEEPROMChangeEx(User access processing)
; Input           : None
; Output	  : C=Normal completion(TRUE:00H)
;                     Noemal end(Block eror)(NMLBLK:01H)
;                     Block nothing finished with elimination(ERRNOBLA:F8H)
;                     With no effective block(ERRNOUSE:FAH)
;                     Device error(ERRDEV:FDH)
;                     Initialization error(ERRINIT:FEH)
; Summary         : If the currently used blocks are full of data, this function searches for the next
; 		  : block to be used and copies data to that block.
;		  : 1.Sets block to be used next.
;		  : 2.Erases block to be used next.
;		  : 3.Transfers the latest data from a valid block to the next block.
;		  : 4.Sets the next block to be used as valid.
;		  : 5.Sets currently valid blocks as invalid.
;-------------------------------------------------------------------
ucEEPROMChangeEx:
	PUSH	HL				; Register evacuation
	PUSH	DE
	PUSH	AX
	
	CALL	!ParamCheck
	CMP	A,#TRUE
	BZ	$EP_CHG_S
	MOV	C,#ERRPRM
	BR	EP_CHG_E
	
EP_CHG_S:
	MOV	A,#0
	MOV	!dFlag,A			;clear flag
	
EP_CHG_10:
	CALL	!ucEEPROMUseBlockSearchEx	;Searches for blocks being used as EEPROM
	CMP	A,#FALSE			;if Abnormal end
	BNZ	$EP_CHG_11			;  else jump
	BR	EP_CHG_NU			;  then jump
	
EP_CHG_11:
	MOV	!dUseBlockTBLNo,A		;Sets block number used

EP_CHG_20:
	MOV	A,!dUseBlockTBLNo		;Use Block table number
	MOV	!dDstBlockTBLNo,A		;Destination Block table number
	CALL	!BlockToAddress			;Address translation
	MOVW	!d2SrcAdr,AX			;Block top address
	
EP_CHG_21:
	MOV	A,!dDstBlockTBLNo		;Destination Block table number
	INC	A
	CMP	A,#EEPROM_BLOCK_NO		;if Last block?
	BC	$EP_CHG_22			;  else loop
	
	MOV	A,#0				;Block table No. = 0
	
EP_CHG_22:
	CMP	A,!dUseBlockTBLNo		;if finish ?
	BNZ	$EP_CHG_23			;  else jump
	BR	EP_CHG_NB			;  then jump
	
EP_CHG_23:
	MOV	!dDstBlockTBLNo,A		;Destination block table number save
	CALL	!BankSet			;Bank set
	CALL	!BlockToAddress			;Address translation
	MOVW	!d2DstAdr,AX			;Destination block top address
	
	MOVW	HL,AX
	CALL	!UseProhibitionBlock		;It investigates in a use prohibition block
	CMP	A,#TRUE				;if Use prohibition block?
	BZ	$EP_CHG_21			;  then jump
	
$IF(AUTOERASE)

EP_CHG_24:
	CALL	!SelfFlashModeOn		;Sets flash memory mode
	CALL	!ucSelfFlashInitialize		;Initializes flash memory
	CMP	A,#TRUE				;if Noemal end
	BZ	$EP_CHG_25			;  then jump
	BR	EP_CHG_IN			;  else jump
	
EP_CHG_25:
	MOV	A,#ERASENUM+1			;erase counter initialization
	MOV	!ci,A
	
EP_CHG_26:
	MOV	A,!dDstBlockTBLNo		;Destination Block table number
	MOV	C,A				;  -->C
	MOVW	HL,#tEEPROM_BANK+0		;bank table
	MOV	A,[HL+C]
	XCH	A,X				;Bank set-->X
	MOVW	HL,#tEEPROM_BLOCK+0		;block table
	MOV	A,[HL+C]			;block no.-->A
	XCH	A,X				;A<-->X
	CALL	!ucSelfFlashBlockErase		;Erases block
	CMP	A,#TRUE				;if Noemal end?
	BZ	$EP_CHG_28			;  then jump
	
	MOV	A,!ci
	DEC	A				;counter-1
	MOV	!ci,A
	BNZ	$EP_CHG_26			;if counter!=0 then jump
	
	MOV	A,!dDstBlockTBLNo		;Block table number set
	CALL	!ucEEPROMUnableEx		;Prohibition of the Brock use
	CMP	A,#TRUE				;if Noemal end?
	BZ	$EP_CHG_27			;  then jump
	BR	EP_CHG_DE			;  else jump
	
EP_CHG_27:
	MOV	A,#1
	MOV	!dFlag,A			;Block eror flag set
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	BR	EP_CHG_21			;loop
	
EP_CHG_28:
	CALL	!SelfFlashModeOff		;Releases flash memory mode

$ENDIF	;AUTOERASE

EP_CHG_30:
	MOVW	AX,!d2DstAdr			;Destiantion block top address
	MOVW	HL,AX
	CALL	!BlankBlock			;It investigates in an eliminated block
	CMP	A,#TRUE				;if eliminated block?
	BNZ	$EP_CHG_21			;  else jump
	
	MOVW	AX,!d2DstAdr			;Destiantion block top address
	ADDW	AX,#DATATOP			;data start offset
	MOVW	!d2SetAdr,AX			;data write address
	
	MOVW	HL,#tEEPROM_BANK+0		;bank table
	MOV	A,!dDstBlockTBLNo		;Destiantion Block table number
	MOV	C,A
	MOV	A,[HL+C]
	MOV	!dBankNo,A			;Bank No. save
	
	MOV	A,#0				;Data No.initialization
	MOV	!ci,A
	
EP_CHG_31:
	MOV	A,!ci
	CMP	A,#DATA_NO_MAX			;if Finish?
	BNC	$EP_CHG_40			;  then jump
	
EP_CHG_32:
	XCH	A,X
	MOV	A,!dUseBlockTBLNo		;A=Block table number
						;X=Data No.
	CALL	!usEEPROMDataSearchEx		;Obtains address of latest data of specified data number
	CMPW	AX,#0000H
	BZ	$EP_CHG_39			;Abnormal completion
	
	MOVW	HL,AX				;HL=read data address
	MOVW	DE,#bDataBuf+0			;DE=buffer address
	
	MOV	C,#0				;Loop Counter
EP_CHG_33:
	MOV	A,[HL]				;read data
	MOV	[DE],A				;storing to buffer
	INCW	HL
	INCW	DE
	
	MOV	A,C
	CMP	A,#WRITE_DATA_LEN		;Loops from 0 to data length
	BNC	$EP_CHG_34			;Loop End
	INC	C				;Loop counter + 1
	BR	$EP_CHG_33			;Loop
	
EP_CHG_34:
	CALL	!SelfFlashModeOn		;Sets flash memory mode
	CALL	!ucSelfFlashInitialize		;Initializes flash memory
	CMP	A,#TRUE				;if Noemal end
	BZ	$EP_CHG_35			;  then jump
	BR	EP_CHG_IN			;  else jump
	
EP_CHG_35:
	MOV	A,#WRITENUM+1			;write counter initialization
	MOV	!cj,A
	MOVW	AX,!d2SetAdr			;Write address
	
EP_CHG_36:
	MOVW	DE,AX				;DE=Write address
	MOV	A,!dDstBlockTBLNo		;destination Block table number
	CALL	!BankSet			;Bank set
	MOV	B,#WRITE_DATA_LEN		;B=byte num
	MOV	A,!dBankNo			;A=Bank No.
	
	CALL	!ucSelfFlashEEPROMWrite		;Writes created write data to EEPROM
	CMP	A,#TRUE				;if Noemal end
	BZ	$EP_CHG_38			;  then jump
	
	CMP	A,#LIB_BLANK_ERR		;if blank error
	BZ	$EP_CHG_37			;  then jump
	
	MOV	A,!cj
	DEC	A				;if retry over
	MOV	!cj,A
	BZ	$EP_CHG_42			;  then jump

EP_CHG_37:
	MOVW	AX,DE
	ADDW	AX,#WRITE_DATA_LEN+1		;Write address update
	BR	$EP_CHG_36
	
EP_CHG_38:
	MOVW	AX,DE
	ADDW	AX,#WRITE_DATA_LEN+1
	MOVW	!d2SetAdr,AX			;Write address update
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	
EP_CHG_39:
	MOV	A,!ci
	INC	A				;Data No.+1
	MOV	!ci,A
	BR	$EP_CHG_31
	
EP_CHG_40:
	CALL	!SelfFlashModeOn		;Sets flash memory mode
	CALL	!ucSelfFlashInitialize		;Initializes flash memory
	CMP	A,#TRUE				;if Noemal end
	BZ	$EP_CHG_41			;  then jump
	BR	EP_CHG_IN			;  else jump
	
EP_CHG_41:
	MOVW	AX,#5555H			;effective flag data 
	MOVW	!bDataBuf+0,AX
	MOVW	!bDataBuf+2,AX
	
	MOVW	AX,!d2DstAdr			;destination address
	MOVW	DE,AX				;Writes start address
	MOV	B,#4-1				;Data number setting
	MOV	A,!dBankNo			;Bank number seetting
	CALL	!ucSelfFlashEEPROMWrite		;Sets block being used as invalid
	CMP	A,#TRUE
	BZ	$EP_CHG_43
	
EP_CHG_42:
	MOV	A,!dDstBlockTBLNo		;Block table number set
	CALL	!ucEEPROMUnableEx		;Prohibition of the Brock use
	CMP	A,#TRUE				;if Noemal end?
	BNZ	$EP_CHG_DE			;  else jump
	
	MOV	A,#1
	MOV	!dFlag,A			;Block eror flag  set
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	BR	EP_CHG_21			;
	
EP_CHG_43:
	MOVW	AX,!d2SrcAdr			;block top address
	ADDW	AX,#4				;address +4
	MOVW	DE,AX				;Writes start address
	MOVW	HL,#tEEPROM_BANK+0
	MOV	A,!dUseBlockTBLNo
	CALL	!BankSet			;Bank set
	MOV	C,A
	MOV	A,[HL+C]			;Bank number seetting
	MOV	B,#4-1				;Data number setting
	CALL	!ucSelfFlashEEPROMWrite		;Sets block being used as invalid
	CMP	A,#TRUE
	BZ	$EP_CHG_44
	
	MOV	A,!dUseBlockTBLNo		;Block table number set
	CALL	!ucEEPROMUnableEx		;Prohibition of the Brock use
	CMP	A,#TRUE				;if Noemal end?
	BNZ	$EP_CHG_DE			;  else jump
	
	MOV	A,#1
	MOV	!dFlag,A			;Block erorflag  set
	
EP_CHG_44:
	MOV	A,!dFlag			;Block erorflag 
	CMP	A,#0				;if Block eror?
	BNZ	$EP_CHG_BL			;  then jump

	MOV	C,#TRUE				;return value=Normal
	BR	$EP_CHG_MO
	
EP_CHG_NU:
	MOV	C,#ERRNOUSE			;Return value = With no effective block (ERRNOUSE)
	BR	$EP_CHG_E
	
EP_CHG_IN:
	MOV	C,#ERRINIT			;Return value = Initialization error (ERRINIT)
	BR	$EP_CHG_MO
	
EP_CHG_NB:
	MOV	C,#ERRNOBLA			;Return value = Block nothing finished with elimination (ERRNOBLA)
	BR	$EP_CHG_E
	
EP_CHG_BL:
	MOV	C,#NMLBLK			;Return value = Block eror (NMLBLK)
	BR	$EP_CHG_MO
	
EP_CHG_DE:
	MOV	C,#ERRDEV			;Return value = Device error (ERRDEV)
	BR	$EP_CHG_MO
	
EP_CHG_MO:
	CALL	!SelfFlashModeOff		;Releases flash memory mode

EP_CHG_E:
	POP	AX				; Register restoration
	POP	DE
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucEEPROMErase (user access processing)
; Input		  : None
; Output	  : C=Normal completion(TRUE:00H)
;                     With no eliminable block(ERRNOERS:F9H)
;                     With no effective block(ERRNOUSE:FAH)
;                     Block eror(ERRBLK:FCH)
;                     Device error(ERRDEV:FDH)
;                     Initialization error(ERRINIT:FEH)
; Summary	  : Erases data in the block to which data is written to next in advance
;-------------------------------------------------------------------
ucEEPROMEraseEx:
	PUSH	HL
	PUSH	DE
	PUSH	AX
	
	CALL	!ParamCheck
	CMP	A,#TRUE
	BZ	$EP_ERASE_00
	MOV	C,#ERRPRM
	BR	$EP_ERASE_E
	
EP_ERASE_00:
	CALL	!ucEEPROMUseBlockSearchEx	;Searches for blocks being used as EEPROM
	CMP	A,#FALSE
	BZ	$EP_ERASE_NU
	
	MOV	!dUseBlockTBLNo,A		;effective Block table number
	MOV	B,A				;B=Processing Block table number
EP_ERASE_10:
	MOV	A,B				;Processing Block table number
	CMP	A,#0				;if first block?
	BNZ	$EP_ERASE_11			;  else jump
	
	MOV	B,#EEPROM_BLOCK_NO		;Last Block table number
EP_ERASE_11:
	DEC	B				;Processing Block table number-1
	
EP_ERASE_12:
	MOV	A,!dUseBlockTBLNo		;effective Block table number
	CMP	A,B				;if finish?
	BZ	$EP_ERASE_NE			;  then jump
	
	MOV	A,B				;Block table number
	CALL	!BankSet			;Bank set
	CALL	!BlockToAddress			;Address translation
	MOVW	HL,AX				;  --> HL
	
	CALL	!UseProhibitionBlock		;It investigates in a use prohibition block
	CMP	A,#TRUE				;if Use prohibition block?
	BZ	$EP_ERASE_10			;  then jump
	
EP_ERASE_13:
	MOV	A,B
	MOV	!dDstBlockTBLNo,A		;erase block table number
	
	CALL	!SelfFlashModeOn		;Sets flash memory mode
	CALL	!ucSelfFlashInitialize		;Initializes flash memory
	CMP	A,#TRUE
	BNZ	$EP_ERASE_IN
	
	MOV	A,#ERASENUM+1			;erase counter initialization
	MOV	!ci,A
	
EP_ERASE_14:
	MOV	A,!dDstBlockTBLNo		;Erase Block table number
	MOV	C,A
	MOVW	HL,#tEEPROM_BANK+0		;bank table
	MOV	A,[HL+C]
	XCH	A,X				;Bank set-->X
	MOVW	HL,#tEEPROM_BLOCK+0		;block table
	MOV	A,[HL+C]			;block no.-->A
	XCH	A,X				;A<-->X
	CALL	!ucSelfFlashBlockErase		;Erases block
	CMP	A,#TRUE				;if Noemal end?
	BZ	$EP_ERASE_TR			;  then jump
	
	MOV	A,!ci
	DEC	A				;counter-1
	MOV	!ci,A
	BNZ	$EP_ERASE_14			;if counter!=0 then jump
	
	MOV	A,!dDstBlockTBLNo		;Block table number set
	CALL	!ucEEPROMUnableEx		;Prohibition of the Brock use
	CMP	A,#TRUE				;if Noemal end?
	BZ	$EP_ERASE_BL			;  else jump
	
EP_ERASE_DE:
	MOV	C,#ERRDEV			;Return value = Device error (ERRDEV)
	BR	$EP_ERASE_MO
	
EP_ERASE_BL:
	MOV	C,#ERRBLK			;Return value = Block eror (ERRBLK)
	BR	$EP_ERASE_MO
	
EP_ERASE_NU:
	MOV	C,#ERRNOUSE			;Return value = With no effective block (ERRNOUSE)
	BR	$EP_ERASE_E
	
EP_ERASE_IN:
	MOV	C,#ERRINIT			;Return value = Initialization error (ERRINIT)
	BR	$EP_ERASE_MO
	
EP_ERASE_NE:
	MOV	C,#ERRNOERS			;Return value = With no eliminable block (ERRNOERS)
	BR	$EP_ERASE_E
	
EP_ERASE_TR:
	MOV	C,#TRUE				;Return value = Normal completion (TRUE)
	
EP_ERASE_MO:
	CALL	!SelfFlashModeOff		;Releases flash memory mode
	
EP_ERASE_E:
	POP	AX
	POP	DE
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucEEPROMGetRemainEx(User access processing)
; Input           : None
; Output	  : The number of data which can be written in
;                   C=Normal(00H`FEH) / Abnormal(FALSE:FFh)
; Summary         : examine the number of the writable data
;-------------------------------------------------------------------
ucEEPROMGetRemainEx:
	PUSH	HL				;Register evacuation
	PUSH	DE
	PUSH	AX
	
	CALL	!ParamCheck
	CMP	A,#TRUE
	BNZ	$EP_GETR_FL
	
EP_GETR_S:
	CALL	!ucEEPROMUseBlockSearchEx	;Searches for blocks being used as EEPROM
	CMP	A,#FALSE			;if Abnormal end
	BZ	$EP_GETR_FL			;  then jump
	
	MOV	C,A
	
	CALL	!BlockToAddress			;Block top address
	MOVW	HL,AX
	
	MOV	A,C
	CALL	!usEEPROMWriteTopSearchEx	;write start address search
	MOVW	DE,AX
	
	MOVW	AX,HL				;Block top address
	ADDW	AX,#DATA_END_OFFSET		;add data end offset
	XCH	A,X				;sub write start address
	SUB	A,E
	XCH	A,X
	SUBC	A,D				;AX=byte num
	
	MOVW	DE,#WRITE_DATA_LEN+1		;length / byte num
	MOV	C,#0				;C=Quotient

EP_GETR_1:
	XCH	A,X
	SUB	A,E
	XCH	A,X
	SUBC	A,D
	BC	$EP_GETR_E
	INC	C
	BR	$EP_GETR_1
	
EP_GETR_FL:
	MOV	C,#FALSE			;Return value = Abnormal end (FALSE)
	
EP_GETR_E:
	POP	AX				;Register restoration
	POP	DE
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucEEPROMUseBlockSearchEx
; Input		  : None
; Output	  : A=Block table number(00H to FEh), Abnormal completion(FALSE:FFH)
; Summary	  : Searches for currently used blocks in flash memory that are allocated to
;		  : EEPROM.
;-------------------------------------------------------------------
ucEEPROMUseBlockSearchEx:
	PUSH	HL
	
	MOV	L,#0				;Initializes loop counter L register
EP_BSRC_T:
	MOV	A,L
	CMP	A,#EEPROM_BLOCK_NO		;Number of loops = Number of EEPROM setting blocks
	BNC	$EP_BSRC_4
	
	CALL	!BankSet			;Bank set
	CALL	!BlockToAddress
	MOVW	!d2SearchAdr,AX
	
	MOV	A,L

	MOV	H,#0	 			;Initializes small loop counter H register
EP_BSRC_1:
	MOV	A,H
	CMP	A,#8
	BNC	$EP_BSRC_2			;Number of loops = 8
	
	MOV	C,A
	MOVW	AX,!d2SearchAdr
	PUSH	HL
	MOVW	HL,AX
	MOV	A,[HL+C]
	POP	HL
	MOV	C,A
	MOV	A,H
	MOV	B,A
	PUSH	HL
	MOV	A,C
	MOVW	HL,#bDataBuf
	MOV	[HL+B],A			;Moves 8 bytes of block start valid/invalid flag to bDataBuf.
	POP	HL
	
	INC	H
	BR	EP_BSRC_1
	
EP_BSRC_2:
	MOVW	AX,!bDataBuf
	CMPW	AX,#5555H
	BNZ	$EP_BSRC_3
	MOVW	AX,!bDataBuf+2
	CMPW	AX,#5555H
	BNZ	$EP_BSRC_3			;When valid flag = 00H,00H,00H,00H
	MOVW	AX,!bDataBuf+4
	CMPW	AX,#0FFFFH
	BNZ	$EP_BSRC_3
	MOVW	AX,!bDataBuf+6
	CMPW	AX,#0FFFFH
	BNZ	$EP_BSRC_3			;and invalid flag = FFH,FFH,FFH,FFH
	
	MOV	A,L				;Return value = Valid block number
	BR	EP_BSRC_E
	
EP_BSRC_3:
	INC	L
	BR	EP_BSRC_T
	
EP_BSRC_4:
	MOV	A,#FALSE			;Return value = Abnormal completion(no valid block)
	
EP_BSRC_E:
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : usEEPROMWriteTopSearchEx
; Input 	  : A=Search block table number
;		  : (block number of block set for EEPROM use by user definition)
; Output	  : AX=Write address(0000H to FFFFH)
; Summary	  : Searches for specified block's write address.
;-------------------------------------------------------------------
usEEPROMWriteTopSearchEx:
	PUSH	HL
	
	CALL	!BankSet			;Bank set
	
	CALL	!BlockToAddress			;
	MOVW	DE,AX				;DE=Block top address
	ADDW	AX,#DATA_END_OFFSET		;add offset
	MOVW	HL,AX				;HL=next address of last data
	MOVW	!d2SearchAdr,AX			;Write address initial value set
	MOVW	AX,DE
	ADDW	AX,#DATATOP
	MOVW	DE,AX				;DE=data top address

EP_WTSR_10:
	MOVW	AX,HL				;search address
	SUBW	AX,#WRITE_DATA_LEN+1		;address-data length
	MOVW	HL,AX
	CMP	A,D				;if finish?
	BC	$EP_WTSR_E			;  then jump
	BNZ	$EP_WTSR_11			;  else jump
	XCH	A,X
	CMP	A,E				;if finish?
	BC	$EP_WTSR_E			;  then jump

EP_WTSR_11:
	MOV	A,[HL+WRITE_DATA_LEN-1]		;read delimiter
	CMP	A,#0FFH				;if delimiter=FFH
	BNZ	$EP_WTSR_E			;  else jump
	
	MOVW	AX,HL
	MOVW	!d2SearchAdr,AX			;Write address update
	BR	$EP_WTSR_10			;loop
	
EP_WTSR_E:
	MOVW	AX,!d2SearchAdr			;return value=Write address
	
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : usEEPROMDataSearchEx
; Input		  : X=Read data number
;		  : A=Currently used block table number
;		  : (block number of block set for EEPROM use by user definition)
; Output	  : AX=Address of latest data(0001H to FFFFH), Abnormal completion(0000H)
; Summary	  : Reads storage address of latest data at specified number.
;-------------------------------------------------------------------
usEEPROMDataSearchEx:
	PUSH	HL

	XCH	A,X
	MOV	B,A				;B=Data No.
	XCH	A,X
	
	CALL	!BankSet			;Bank set

	CALL	!BlockToAddress			;
	MOVW	DE,AX				;DE=Block top address
	ADDW	AX,#DATA_END_OFFSET		;last address offset
	MOVW	HL,AX				;HL=next address of last data
	MOVW	AX,DE
	ADDW	AX,#DATATOP
	MOVW	DE,AX				;DE=data top address

EP_DASR_10:
	MOVW	AX,HL				;search adddress
	SUBW	AX,#WRITE_DATA_LEN+1		;address-length
	MOVW	HL,AX
	CMP	A,D				;if finish?
	BC	$EP_DASR_ND			;  then jump
	BNZ	$EP_DASR_11			;  else jump
	XCH	A,X
	CMP	A,E				;if finish?
	BC	$EP_DASR_ND			;  then jump

EP_DASR_11:
	MOV	A,[HL+WRITE_DATA_LEN-1]		;read delimiter
	CMP	A,#00H				;if delimiter=00H
	BNZ	$EP_DASR_10			;  else jump
	
	MOV	A,[HL]				;read Data No.
	CMP	A,B				;if Data No. match
	BNZ	$EP_DASR_10			;  else jump
	
EP_DASR_TR:
	MOVW	AX,HL				;return value=newest data address
	BR	$EP_DASR_E
	
EP_DASR_ND:
	MOVW	AX,#0				;return value=0
	
EP_DASR_E:
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucEEPROMUnableEx
; Input           : A=Block table number
; Output          : A=Normal completion(TRUE:0x00) / Abnormal completion(FALSE:0xff)
; Summary         : make a block the ban on use
;-------------------------------------------------------------------
ucEEPROMUnableEx:
	PUSH	HL				; Register evacuation
	
EP_UNAB_S:
	MOV	C,A
	CALL	!BankSet			;Bank set
	
	CALL	!BlockToAddress			;Block top address
	MOVW	DE,AX				;  -->DE
	
	MOVW	HL,#tEEPROM_BANK+0
	MOV	A,[HL+C]			;Bank number seetting
	MOV	C,A

	MOVW	HL,#bDataBuf+0			;buffer address
	MOV	A,#00H				;write data=00H
	MOV	[HL+0],A
	MOV	[HL+1],A
	MOV	[HL+2],A
	MOV	[HL+3],A
	
	MOV	H,#1				;error flag 
	MOV	L,#WRITENUM+1			;write counter
EP_UNAB_1:
	PUSH	BC
	MOV	B,#4				;B=data byte num
	MOV	A,C				;A=Bank No.
	CALL	!ucSelfFlashWordWrite		;Writes created write data to EEPROM
	POP	BC
	CMP	A,#TRUE				;if Noemal end
	BNZ	$EP_UNAB_1A			;  then jump
	
	MOV	H,#0				;error flag reset
	BR	EP_UNAB_2
	
EP_UNAB_1A:
	DEC	L				;write counter-1
	BNZ	$EP_UNAB_1			;loop
	
EP_UNAB_2:
	MOVW	AX,DE
	ADDW	AX,#4
	MOVW	DE,AX
	MOV	L,#WRITENUM+1			;write counter
EP_UNAB_3:
	PUSH	BC
	MOV	A,C				;A=Bank No.
	MOV	B,#4				;B=data byte num
	CALL	!ucSelfFlashWordWrite		;Writes created write data to EEPROM
	POP	BC
	CMP	A,#TRUE				;if Noemal end
	BNZ	$EP_UNAB_3A			;  then jump
	
	MOV	H,#0				;error flag reset
	BR	EP_UNAB_4
	
EP_UNAB_3A:
	DEC	L				;write counter-1
	BNZ	$EP_UNAB_3			;loop
	
EP_UNAB_4:
	MOV	A,H
	CMP	A,#0				;if Noemal end?
	BZ	$EP_UNAB_TR			;  then jump
	
EP_UNAB_FL:
	MOV	A,#FALSE			;return value=Abnormal end(FALSE)
	BR	$EP_UNAB_E			;jump to end
	
EP_UNAB_TR:
	MOV	A,#TRUE				;return value=Noemal end(TRUE)
	
EP_UNAB_E:
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : SelfFlashModeOn
; Input		  : None
; Output	  : None
; Summary	  : Sets self rewriting mode(A1).
;-------------------------------------------------------------------
SelfFlashModeOn:
	DI
	CALL	!_FlashStart
	EI
	RET


;-------------------------------------------------------------------
; Processing name : SelfFlashModeOff
; Input		  : None
; Output	  : None
; Summary	  : Releases self rewriting mode.
;-------------------------------------------------------------------
SelfFlashModeOff:
	DI
	CALL	!_FlashEnd
	EI
	RET


;-------------------------------------------------------------------
; Processing name : ucSelfFlashInitialize
; Input		  : None
; OutPut	  : A=Normal completion(TRUE:00H) / Abnormal completion(FALSE:FFH)
; Summary	  : Initializes self programming firmware for EEPROM emulation
;		  :  Initializes flash firmware
;		  :  Checks FLMD pin[programming may be disabled, depending on the FLMD pin's
;		  :  status]
;-------------------------------------------------------------------
ucSelfFlashInitialize:
	MOVW	AX,#b100EntryRAM
	CALL	!_FlashEnv			;Entry RAM initialization setting
	
	CALL	!_CheckFLMD			;Checks FLMD pin status
	CMP	A,#LIB_NORMAL_END
	BZ	$SF_INIT_TR			;Normal completion
	
	MOV	A,#FALSE			;Abnormal completion
	BR	SF_INIT_E			;Return value = Abnormal completion (FALSE)
	
SF_INIT_TR:
	MOV	A,#TRUE				;Return value = Normal completion (TRUE)
	
SF_INIT_E:
	RET


;-------------------------------------------------------------------
; Processing name : ucSelfFlashBlockErase
; Input		  : A=Erase bank number
;		  : X=Erase block number
; OutPut	  : A=Normal completion(TRUE:00H) / Abnormal completion(FALSE:FFH)
; Summary	  : Erases specified block.
;-------------------------------------------------------------------
ucSelfFlashBlockErase:
	PUSH	HL
	
	MOVW	HL,AX
	
SF_ERS_T:
	MOVW	AX,HL
	XCH	A,X
	MOV	B,A				;Block number setting
	XCH	A,X				;Bnak number setting
	DI
	CALL	!_FlashBlockBlankCheck		;Flash memory blank check library call
	EI
	
	CMP	A,#LIB_BLKCHK_ERR
	BNZ	$SF_ERS_1
	
	MOVW	AX,HL
	XCH	A,X
	MOV	B,A				;Block number setting
	XCH	A,X				;Bnak number setting
	DI
	CALL	!_FlashBlockErase		;Flash memorry erase library call
	EI
	
	CMP	A,#LIB_NORMAL_END
	BZ	$SF_ERS_T
	CMP	A,#LIB_STATE_INT
	BZ	$SF_ERS_T
	BR	SF_ERS_FL
	
SF_ERS_1:
	CMP	A,#LIB_NORMAL_END
	BZ	$SF_ERS_TR
	
	CMP	A,#LIB_STATE_INT
	BZ	$SF_ERS_T
	
SF_ERS_FL:					;Abnormal completion
	MOV	A,#FALSE			;Return value = Abnormal completion (FALSE)
	BR	SF_ERS_E
	
SF_ERS_TR:					;Normal completion
	MOV	A,#TRUE				;Return value = normal completion (TRUE)
	
SF_ERS_E:
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucSelfFlashEEPROMWrite
; Input		  : A=Bank number
;		  : B=Data number
;		  : DE=Writes start address
; Output	  : A=Normal completion(TRUE:00H) / Abnormal completion(FALSE:FFH)
; Summary	  : EEPROM to write data.
;-------------------------------------------------------------------
ucSelfFlashEEPROMWrite:
	PUSH	HL
	
	MOV	!WriteBnak,A			;Bank number setting
	MOVW	AX,DE
	MOVW	!WriteAddress,AX		;Write start address setting
	
SF_EEW_T:
	MOV	A,B				;Data number setting
	ROR	A,1
	ROR	A,1
	AND	A,#03FH
	INC	A
	MOV	B,A				;(Data number/4)+1 ( word <- byte )
	
	MOVW	AX,#WordAddr
	MOVW	HL,#bDataBuf
	
	DI
	CALL	!_EEPROMWrite			;EEPROM write library call
	EI
	
	CMP	A,#LIB_NORMAL_END
	BZ	$SF_EEW_TR
	CMP	A,#LIB_STATE_INT
	BZ	$SF_EEW_T
	
SF_EEW_FL:					;Abnormal completion
;	MOV	A,#FALSE			;Return value = Abnormal completion (FALSE)
	BR	SF_EEW_E
	
SF_EEW_TR:					;Normal completion
	MOV	A,#TRUE				;Return value = normal completion (TRUE)
	
SF_EEW_E:
	POP	HL
	RET


;-------------------------------------------------------------------
; Processing name : ucSelfFlashWordWrite
; Input		  : A=Bank number
;		  : B=Data number
;		  : DE=Writes start address
; Output	  : A=Normal completion(TRUE:00H) / Abnormal completion(FALSE:FFH)
; Summary	  : write word data to EEPROM.
;-------------------------------------------------------------------
ucSelfFlashWordWrite:
	PUSH	HL
SF_WW_S:
	
	MOV	!WriteBnak,A			;Bank number setting
	MOVW	AX,DE
	MOVW	!WriteAddress,AX		;Write start address setting
	
SF_WW_T:
	MOV	A,B				;Data number setting
	DEC	A
	ROR	A,1
	ROR	A,1
	AND	A,#03FH
	INC	A
	MOV	B,A				;((Data number-1)/4)+1 ( word <- byte )
	
	MOVW	AX,#WordAddr
	MOVW	HL,#bDataBuf
	
	DI
	CALL	!_FlashWordWrite		;Word write library call
	EI
	
	CMP	A,#LIB_NORMAL_END
	BZ	$SF_WW_TR
	CMP	A,#LIB_STATE_INT
	BZ	$SF_WW_T
	
SF_WW_FL:					;Abnormal completion
	MOV	A,#FALSE			;Return value = Abnormal completion (FALSE)
	BR	SF_EEW_E
	
SF_WW_TR:					;Normal completion
	MOV	A,#TRUE				;Return value = normal completion (TRUE)
	
SF_WW_E:
	POP	HL
	RET

;--------------------------------------------------------------------
; Processing name : BlockToAddress
; Input		  : A=Block number
; Output	  : AX=Block start address
; Processing	  : A block number is calculated from block number.
;--------------------------------------------------------------------
BlockToAddress:
	PUSH	HL
	
	MOV	C,A
	MOVW	HL,#tEEPROM_BLOCK
	MOV	A,[HL+C]
	
	MOV	X,#0
	CLR1	CY
	ROLC	A,1
	ROLC	A,1
	
	POP	HL
	
	RET


;--------------------------------------------------------------------
; Processing name : WriteDataLenght
; Input		  : A=Number of data
; Output	  : A=Write data length
; Processing	  : Calculate write data length from number of data
;		  : A = [ ((( A+1 )/4 )+1 )*4 ]
;--------------------------------------------------------------------
WriteDataLenght:
	INC	A
	ROR	A,1
	ROR	A,1
	AND	A,#00111111B
	INC	A
	ROL	A,1
	ROL	A,1
	AND	A,#11111100B
	DEC	A
	
	RET


;--------------------------------------------------------------------
; Processing name : UseProhibitionBlock
; Input		  : HL=effective/invalid flag address
; Output	  : A=TRUE/FALSE
; Processing	  : It investigates in a use prohibition block
;--------------------------------------------------------------------
UseProhibitionBlock:
	PUSH	HL
UPB_S:
	MOV	A,[HL+0]			; effective flag =00000000H?
	CMP	A,#00H
	BNZ	$UPB_1
	MOV	A,[HL+1]
	CMP	A,#00H
	BNZ	$UPB_1
	MOV	A,[HL+2]
	CMP	A,#00H
	BNZ	$UPB_1
	MOV	A,[HL+3]
	CMP	A,#00H
	BZ	$UPB_3
UPB_1:
	MOV	A,[HL+4]			; flag =00000000H?
	CMP	A,#00H
	BNZ	$UPB_2
	MOV	A,[HL+5]
	CMP	A,#00H
	BNZ	$UPB_2
	MOV	A,[HL+6]
	CMP	A,#00H
	BNZ	$UPB_2
	MOV	A,[HL+7]
	CMP	A,#00H
	BZ	$UPB_3
UPB_2:
	MOV	A,#FALSE
	BR	$UPB_E
UPB_3:
	MOV	A,#TRUE
UPB_E:
	POP	HL
	RET


;--------------------------------------------------------------------
; Processing name : EffectiveBlock
; Input		  : HL=effective/invalid flag address
; Output	  : A=TRUE/FALSE
; Processing	  : It investigates in an effective block
;--------------------------------------------------------------------
EffectiveBlock:
	PUSH	HL
EFB_S:
	MOV	A,[HL+0]			; effective flag =55555555H?
	CMP	A,#055H
	BNZ	$EFB_2
	MOV	A,[HL+1]
	CMP	A,#055H
	BNZ	$EFB_2
	MOV	A,[HL+2]
	CMP	A,#055H
	BNZ	$EFB_2
	MOV	A,[HL+3]
	CMP	A,#055H
	BNZ	$EFB_2

	MOV	A,[HL+4]			; flag =FFFFFFFFH?
	CMP	A,#0FFH
	BNZ	$EFB_2
	MOV	A,[HL+5]
	CMP	A,#0FFH
	BNZ	$EFB_2
	MOV	A,[HL+6]
	CMP	A,#0FFH
	BNZ	$EFB_2
	MOV	A,[HL+7]
	CMP	A,#0FFH
	BNZ	$EFB_2
EFB_1:
	MOV	A,#TRUE
	BR	$EFB_E
EFB_2:
	MOV	A,#FALSE
EFB_E:
	POP	HL
	RET


;--------------------------------------------------------------------
; Processing name : BlankBlock
; Input		  : HL=effective/invalid flag address
; Output	  : A=TRUE/FALSE
; Processing	  : It investigates in an eliminated block
;--------------------------------------------------------------------
BlankBlock:
	PUSH	HL
BRB_S:
	MOV	A,[HL+0]			; effective flag =FFFFFFFFH?
	CMP	A,#0FFH
	BNZ	$BRB_2
	MOV	A,[HL+1]
	CMP	A,#0FFH
	BNZ	$BRB_2
	MOV	A,[HL+2]
	CMP	A,#0FFH
	BNZ	$BRB_2
	MOV	A,[HL+3]
	CMP	A,#0FFH
	BNZ	$BRB_2

	MOV	A,[HL+4]			; flag =FFFFFFFFH?
	CMP	A,#0FFH
	BNZ	$BRB_2
	MOV	A,[HL+5]
	CMP	A,#0FFH
	BNZ	$BRB_2
	MOV	A,[HL+6]
	CMP	A,#0FFH
	BNZ	$BRB_2
	MOV	A,[HL+7]
	CMP	A,#0FFH
	BNZ	$BRB_2
BRB_1:
	MOV	A,#TRUE
	BR	$BRB_E
BRB_2:
	MOV	A,#FALSE
BRB_E:
	POP	HL
	RET


;--------------------------------------------------------------------
; Processing name : BankSet
; Input		  : A=bank table no.
; Output	  : none
; Processing	  : Bank register (=BANK) set
;--------------------------------------------------------------------
BankSet:

$IF(BANKUSE)
	PUSH	HL
	MOV	C,A
	MOVW	HL,#tEEPROM_BANK+0
	MOV	A,[HL+C]		;Bank number set
	MOV	BANK,A
	MOV	A,C
	POP	HL
$ENDIF
	RET


;--------------------------------------------------------------------
; Processing name : ParamCheck
; Input		  : none
; Output	  : A=TRUE/FALSE
; Processing	  : check parameteres
;--------------------------------------------------------------------
ParamCheck:
	MOV	A,#DATALENGTH
	CMP	A,#1
	BC	$PRC_2
	CMP	A,#255
	BNC	$PRC_2
	
	MOV	A,#DATANOMAX
	CMP	A,#254
	BNC	$PRC_2
	
	MOV	A,#WRITENUM
	CMP	A,#254
	BNC	$PRC_2
	
	MOV	A,#ERASENUM
	CMP	A,#255
	BNC	$PRC_2
	
PRC_1:
	MOV	A,#TRUE
	BR	$PRC_E
PRC_2:
	MOV	A,#FALSE
PRC_E:
	RET

	END

