;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;   System	    : EEPROM emulation Sample program
;   File name       : USER_MAINex.asm
;   Version name    : 1.01
;   Target CPU      : uPD78F0547(78K0/KF2)
;   Last updated    : 2005/10/17
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

EXTRN	ucEEPROMInitEx
EXTRN	ucEEPROMWriteEx
EXTRN	ucEEPROMReadEx
EXTRN	ucEEPROMEraseEx
EXTRN	ucEEPROMChangeEx

TRUE	EQU	00H
FALSE	EQU	0FFH
NMLBLK	EQU	01H
ERRFULL	EQU	0F5H

;---------------------------------------------------------------------
;	Stores stack
;---------------------------------------------------------------------
DSTACK	DSEG	AT	0FB00H
		DS	80H	; STACK AREA
STACKINI:

;---------------------------------------------------------------------
;	Sets interrupt vector table
;---------------------------------------------------------------------
VCTTBL00	CSEG	AT 0000H
				;addr
	DW	MAIN		; 00H

VCTTBL04	CSEG	AT 0004H

	DW	MAIN		; 04H
	DW	MAIN		; 06H
	DW	MAIN		; 08H
	DW	MAIN		; 0AH
	DW	MAIN		; 0CH
	DW	MAIN		; 0EH
	DW	MAIN		; 10H
	DW	MAIN		; 12H
	DW	MAIN		; 14H
	DW	MAIN		; 16H
	DW	MAIN		; 18H
	DW	MAIN		; 1AH
	DW	MAIN		; 1CH
	DW	MAIN		; 1EH
	DW	MAIN		; 20H
	DW	MAIN		; 22H
	DW	MAIN		; 24H
	DW	MAIN		; 26H
	DW	MAIN		; 28H
	DW	MAIN		; 2AH
	DW	MAIN		; 2CH
	DW	MAIN		; 2EH
	DW	MAIN		; 30H
	DW	MAIN		; 32H
	DW	MAIN		; 34H
	DW	MAIN		; 36H
	DW	MAIN		; 38H
	DW	MAIN		; 3AH

;---------------------------------------------------------------------
;	Sets data passing buffer to EEPROM emulation
;---------------------------------------------------------------------
U_DATA	DSEG
EEPROMBuf:	DS	256
CHANGEFL:	DS	1
		DS	1
		
U_PROG	CSEG
;---------------------------------------------------------------------
;	Sample program
;---------------------------------------------------------------------
MAIN:
	DI
	SEL	RB0			;Register bank 0

	MOVW	AX,#STACKINI
	MOVW	SP,AX			;Sets stack pointer

	MOV	IMS,#0CCH
	MOV	IXS,#00H		;1024byte

	MOV	PCC,#00H		;Selects clock(division ratio) -> IN: 10MHz = OUT: 10MHz

	CLR1	EXCLK			;Select mode of fx operation -> EXCLK=0, OSCSEL=1
	SET1	OSCSEL
	MOV	OSTS,#05H
	CLR1	MSTOP
MAIN_00:
	NOP
	BF	OSTC.0,$MAIN_00
	SET1	XSEL
	SET1	MCM0
MAIN_01:
	NOP
	BF	MCS,$MAIN_01


	MOV	LVIM,#00H		;Disables low voltage detection
	MOV	LVIS,#00H

	EI

	CALL	!EEPROM_TEST

MAIN_LOOP:
	BR	MAIN_LOOP


;---------------------------------------------------------------------
;	EEPROM Read/Write test program
;---------------------------------------------------------------------
EEPROM_TEST:

	CALL	!ucEEPROMInitEx		;Initial setting processing
	
	MOV	A,C			;get a return value
	CMP	A,#TRUE			;if Normal end?
	BZ	$EP_TST_1		;  then jump
	
	CMP	A,#NMLBLK		;if Normal(but block error)?
	BZ	$EP_TST_1		;  then jump
	
	BR	EP_TST_FL
	
EP_TST_1:
	MOV	A,#0
	MOV	!CHANGEFL,A		;initialize a block change flag
	
	MOV	A,#11H
	MOV	!EEPROMBuf,A
	MOV	A,#22H
	MOV	!EEPROMBuf+1,A
	MOV	A,#33H
	MOV	!EEPROMBuf+2,A
	MOV	A,#44H
	MOV	!EEPROMBuf+3,A
	
EP_TST_2:
	MOVW	AX,#EEPROMBuf+0
	MOVW	DE,AX			;Sets buffer address to DE register
	MOV	X,#0			;Sets write data number to X register
	
	CALL	!ucEEPROMWriteEx	;Calls EEPROM write processing
	
	MOV	A,C
	CMP	A,#TRUE			;Checks that return value (C register) is "Normal completion"
	BZ	$EP_TST_5
	
	CMP	A,#ERRFULL		;if not data full
	BNZ	$EP_TST_FL		;  then jump
	
EP_TST_3:
	CALL	!ucEEPROMChangeEx	;block change processing
	
	MOV	A,C			;get a return value
	CMP	A,#TRUE			;if Normal end?
	BZ	$EP_TST_4		;  then jump
	
	CMP	A,#NMLBLK		;if Normal end(but block error)?
	BNZ	$EP_TST_FL		;  else jump
	
EP_TST_4:
	MOV	A,#1
	MOV	!CHANGEFL,A		;set block change flag
	BR	$EP_TST_2
	
EP_TST_5:
	MOV	A,#00H
	MOV	!EEPROMBuf,A
	MOV	!EEPROMBuf+1,A
	MOV	!EEPROMBuf+2,A
	MOV	!EEPROMBuf+3,A

	MOVW	AX,#EEPROMBuf
	MOVW	DE,AX			;Sets buffer address to DE register
	MOV	X,#0			;Sets read data number to X register
	
	CALL	!ucEEPROMReadEx		;Calls EEPROM read processing
	
	MOV	A,C
	CMP	A,#TRUE			;Checks that return value (C register) is "Normal completion"
	BNZ	$EP_TST_FL
	
EP_TST_6:
	MOV	A,!EEPROMBuf
	CMP	A,#11H
	BNZ	$EP_TST_FL
	MOV	A,!EEPROMBuf+1
	CMP	A,#22H
	BNZ	$EP_TST_FL 
	MOV	A,!EEPROMBuf+2
	CMP	A,#33H
	BNZ	$EP_TST_FL 
	MOV	A,!EEPROMBuf+3
	CMP	A,#44H
	BNZ	$EP_TST_FL 		;Compares to judge whether read data is correct

EP_TST_7:
	MOV	A,!CHANGEFL		;check block chnage flag
	CMP	A,#0			;if do not change a block?
	BZ	$EP_TST_TR		;  then jump
	
	CALL	!ucEEPROMEraseEx
	
	MOV	A,C
	CMP	A,#TRUE			;Checks that return value (C register) is "Normal completion"
	BNZ	$EP_TST_FL
	
EP_TST_TR:
	MOV	C,#TRUE			;Return value = Normal completion
	BR	EP_TST_E
	
EP_TST_FL:
	MOV	C,#FALSE		;Return value = Abnormal completion
	
EP_TST_E:
	RET

	END

