/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  System       : EEPROM emulation
  Module name  : Sammple Main program
  File name    : MainEx.c
  Version name : 1.01
  Target CPU   : uPD78F0547(78K0/KF2)
  Last updated : 2005/10/17
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*--- Expanded functions -------------------------------------------*/
#pragma	sfr
#pragma	di
#pragma	ei
#pragma	nop
#pragma asm

/*--- Type declarations --------------------------------------------*/
typedef	unsigned char  UCHAR;
typedef	unsigned short USHORT;
typedef	unsigned long  ULONG;
typedef			 char  CHAR;
typedef			 short SHORT;
typedef			 long  LONG;

#define	TRUE	(0x00)
#define	FALSE	(0xFF)
#define ERROR	(0xEE)
#define	NMLBLK	(0x01)
#define	ERRFULL	(0xF5)

/*--- Prototype declarations ---------------------------------------*/
void main( void );
UCHAR EEPROMTest( void );

extern	UCHAR  ucEEPROMInitEx( void );
extern	UCHAR  ucEEPROMReadEx( UCHAR ucDataNo , UCHAR *pDataAddress );
extern	UCHAR  ucEEPROMWriteEx( UCHAR ucDataNo, UCHAR *pDataAddress );
extern	UCHAR  ucEEPROMChangeEx( void );
extern	UCHAR  ucEEPROMEraseEx( void );
extern	UCHAR  ucEEPROMGetRemainEx( void );

	UCHAR ucEEPROMBuf[10];

void main( void )
{
	UCHAR	ucStatus;
	
	IMS	= 0xCC;
	IXS	= 0x00;				/* 6144byte */	
	
	PCC	= 0x00;				/* Clock select(division ratio) -> IN: 5MHZ = OUT: 5MHZ */
	
	EXCLK = 0;				/* Select mode of fx operation -> EXCLK=0, OSCSEL=1*/
	OSCSEL = 1;
	OSTS = 0x05;			/* Oscillation stabilization time */
	MSTOP = 0;				/* MOC.bit7:X1 oscillator operation	*/
	while( OSTC.0 == 0);
	XSEL = 1;
	MCM0 = 1;
	while( MCS == 0 );

	LVIM = 0x00;			/* Prohibits low voltage detection */
	LVIS = 0x00;
	
	ucStatus = EEPROMTest();			/* EEPROM test program */
	
	while(1){
	}
}

/*----- EEPROM test program ------------------------------------------------*/
UCHAR EEPROMTest( void )				/* EEPROM test program main			*/
{
	UCHAR	ucEEPROMNo;					/* EEPROM data number 				*/
	UCHAR	ucResult;					/* processing result				*/
	UCHAR	ucChange;					/* block change flag 				*/
	
	/*----- Initial processing -----*/
	ucResult = ucEEPROMInitEx();		/* EEPROM emulation initial setting function	*/
	if (( ucResult != TRUE ) &&
		( ucResult != NMLBLK)){			/* Abnormal end? 					*/
		return( ERROR );				/* return an error					*/
	}
	ucChange = FALSE;					/* initialize a block change flag	*/
	
	/*----- Data write processing -----*/
	ucEEPROMNo = 0;						/* set 0 to data number				*/
	ucEEPROMBuf[0] = 0x11;				/* set write data					*/
	ucEEPROMBuf[1] = 0x22;
	ucEEPROMBuf[2] = 0x33;
	ucEEPROMBuf[3] = 0x44;
	
	while (1) {							/* do a loop in consideration of data full case	*/
										/* EEPROM data write function		*/
		ucResult = ucEEPROMWriteEx( ucEEPROMNo, &ucEEPROMBuf[0] );
		if ( ucResult == TRUE ){		/* Normal end? 						*/
			break;						/* exit a loop		 				*/
		}
		if ( ucResult != ERRFULL ){		/* Abnormal end? 					*/
			return( ERROR );			/* return an error					*/
		}
		ucResult = ucEEPROMChangeEx();	/* EEPROM block change function		*/
		if (( ucResult != TRUE ) &&	
			( ucResult != NMLBLK )){	/* Abnormal end? 					*/
			return( ERROR );			/* return an error					*/
		}
		ucChange = TRUE;				/* set a block change flag 			*/
	}
	
	/*----- f[^ǂݍݏ -----*/
	ucEEPROMBuf[0] = 0x00;				/* clear an area with 0				*/
	ucEEPROMBuf[1] = 0x00;				/* (not necessary to clear it)		*/
	ucEEPROMBuf[2] = 0x00;
	ucEEPROMBuf[3] = 0x00;
	
	ucEEPROMNo = 0;						/* Data number						*/
										/* EEPROM data read function 		*/
	ucResult = ucEEPROMReadEx( ucEEPROMNo, &ucEEPROMBuf[0] );
	if ( ucResult != TRUE ){			/* Abnormal end? 					*/
		return( ERROR );				/* return an error					*/
	}
	
	if (( ucEEPROMBuf[0] != 0x11 )||
		( ucEEPROMBuf[1] != 0x22 )||
		( ucEEPROMBuf[2] != 0x33 )||
		( ucEEPROMBuf[3] != 0x44 )){	/* case of data disagreement		*/
		return( FALSE );				/* return an error					*/
	}
	
	/*----- ubN -----*/
	if ( ucChange == TRUE ){			/* block change? 					*/
		ucResult = ucEEPROMEraseEx();	/* EEPROM block erase function		*/
		if ( ucResult != TRUE ){		/* Abnormal end? 					*/
			return( ERROR );			/* return an error					*/
		}
	}
	
	return( TRUE );						/* Normal end 						*/
}

